//+------------------------------------------------------------------+
//|                                                 MACD colored.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
//mod + sto  + baseMAPeriod    ki

#property copyright "mladen"
#property link      "mladenfx@gmail.com"

enum enMacdType
{
   macd_onReg,    // Regular macd
   macd_onZlag,   // Macd based on zero lag ma
   macd_onRsi,    // Macd based on Rsi
   macd_onMom,    // Macd based on Momentum
   macd_onCci,    // Macd based on Cci
   macd_onSto     // Macd based on Stochastic
};

#property indicator_separate_window
#property indicator_buffers  7
#property indicator_color1   clrGreen
#property indicator_color2   clrGreen
#property indicator_color3   clrRed
#property indicator_color4   clrRed
#property indicator_color5   clrGray
#property indicator_color6   clrGold
#property indicator_color7   clrDimGray
#property indicator_width1   1
#property indicator_width2   2
#property indicator_width3   1
#property indicator_width4   2
#property indicator_width5   2
#property indicator_width6   1

extern string             _              = "MACD settings";
extern int                FastEMA        = 12;
extern int                SlowEMA        = 26;
extern int                SignalEMA      = 9;
extern ENUM_APPLIED_PRICE PriceType      = PRICE_CLOSE;
extern bool               ShowHisto      = true;
extern bool               ShowOSMA       = false;
extern int                OsmaMultiplier = 2;
extern bool               ShowLines      = true; //if falce - osma autoscale
input enMacdType          MACDType       = macd_onReg;
extern int                RSIPeriod      = 14;
extern int                MOMPeriod      = 14;
extern int                CCIPeriod      = 10;
extern int                StoK_period    = 14;
extern int                StoD_period    = 3;
extern int                StoSl_period   = 3;
extern int                StoPriceField  = 0;
extern int                StoMain_MaMode = 0;
extern int                StoSig_MaMode  = 0;
extern int                baseMaPeriod   = 1;
extern int                baseMaMode     = 0;

//
//
//
//
//

extern string TimeFrame         = "Current time frame";
extern string  __               = "alerts";
extern bool    alertsOn         = true;
extern bool    MACDalerts       = true;
extern bool    OsMAalerts       = true;
extern bool    alertsMessage    = true;
extern bool    alertsSound      = true;
extern bool    alertsEmail      = false;
extern string  soundfile        = "news.wav";     //"alert2.wav"
extern string  ___              = "arrows";
input bool     arrowsVisible    = false;                 // Arrows visible true/false?
input bool     arrowsOnNewest   = false;                 // Arrows drawn on newest bar of higher time frame bar true/false?
input string   arrowsIdentifier = "macd Arrows1";        // Unique ID for arrows
input double   arrowsUpperGap   = 0.5;                   // Upper arrow gap
input double   arrowsLowerGap   = 0.5;                   // Lower arrow gap
input color    arrowsUpColor    = clrBlue;               // Up arrow color
input color    arrowsDnColor    = clrCrimson;            // Down arrow color
input int      arrowsUpCode     = 233;                   // Up arrow code
input int      arrowsDnCode     = 234;                   // Down arrow code
input int      arrowsUpSize     = 2;                     // Up arrow size
input int      arrowsDnSize     = 2;                     // Down arrow size

extern string  note_MACD_Types_ = "0 regularMACD-ema 1ZeroLagMACD 2MACDofRSI  3MACDofMomentum 4MACDofCCI 5MACDofStochastic";
extern string  Sto_PriceField_  = "0Low/High  1Close/Close";

//
//
//
//
//

double buffer1[];
double buffer2[];
double buffer3[];
double buffer4[];
double buffer5[];
double buffer6[];
double buffer7[];
double buffer8[],slope[];
string IndicatorName;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int init()
{
   IndicatorBuffers(9);
   SetIndexBuffer(0,buffer1);
   SetIndexBuffer(1,buffer2);
   SetIndexBuffer(2,buffer3);
   SetIndexBuffer(3,buffer4);
   SetIndexBuffer(4,buffer5);
   SetIndexBuffer(5,buffer6);
   SetIndexBuffer(6,buffer7);
   SetIndexBuffer(7,buffer8);
   SetIndexBuffer(8,slope);
   
   if (ShowOSMA)
   {
         ShowHisto = true;
         SetIndexStyle(6,DRAW_LINE);
   }         
   else  SetIndexStyle(6,DRAW_NONE);
   if (ShowHisto)
   {
      SetIndexStyle(0,DRAW_HISTOGRAM);
      SetIndexStyle(1,DRAW_HISTOGRAM);
      SetIndexStyle(2,DRAW_HISTOGRAM);
      SetIndexStyle(3,DRAW_HISTOGRAM);
   }
   else
   {
      SetIndexStyle(0,DRAW_NONE);
      SetIndexStyle(1,DRAW_NONE);
      SetIndexStyle(2,DRAW_NONE);
      SetIndexStyle(3,DRAW_NONE);
   }      
   
   if (ShowLines)
   {
         SetIndexStyle(4,DRAW_LINE);
         SetIndexStyle(5,DRAW_LINE);
   }         
   else  
   {
         SetIndexStyle(4,DRAW_NONE);
         SetIndexStyle(5,DRAW_NONE);
   }
   //
   //
   //
   //
   //
   
   timeFrame = stringToTimeFrame(TimeFrame);
   string ShortName;
   if  (MACDType==macd_onZlag)
         ShortName = timeFrameToString(timeFrame)+" ZeroLag MACD (";
   else  ShortName = timeFrameToString(timeFrame)+" MACD (";
         ShortName = ShortName+(string)FastEMA+","+(string)SlowEMA+","+(string)SignalEMA+")";
         switch (MACDType)
         {
            case macd_onRsi: ShortName = ShortName+" of RSI       ("+(string)RSIPeriod+")"; break;
            case macd_onMom: ShortName = ShortName+" of Momentum  ("+(string)MOMPeriod+")"; break;
            case macd_onCci: ShortName = ShortName+" of CCI       ("+(string)CCIPeriod+")"; break;
            case macd_onSto: ShortName = ShortName+" of Stochastic("+(string)StoK_period+","+(string)StoD_period+","+(string)StoSl_period+")"; break;
 
         }
   IndicatorSetString(INDICATOR_SHORTNAME,ShortName);

   //
   //
   //
   //
   //

   IndicatorName = WindowExpertName();
   return(0);
}
int deinit()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   double signalAlpha = 2.0/(1.0+SignalEMA);
   int  i,counted_bars  = IndicatorCounted();
   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
      int limit = MathMin(Bars-counted_bars,Bars-1);
      
   //
   //
   //
   //
   //

   if (_ == "DoingZeroLag")  { CalculateZeroLag(limit); return(0); }
   
   //
   //
   //
   //
   //
   if (_ == "calculating")
   {            
     for(i = limit; i >= 0 ; i--)
     {
        switch (MACDType)
        {
           case macd_onRsi: buffer8[i] = iRSI(NULL,0,RSIPeriod,PriceType,i);      break;
           case macd_onMom: buffer8[i] = iMomentum(NULL,0,MOMPeriod,PriceType,i); break;
           case macd_onCci: buffer8[i] = iCCI(NULL,0,CCIPeriod,PriceType,i);      break;
           case macd_onSto: buffer8[i] = iStochastic(NULL,0,StoK_period,StoD_period,StoSl_period,StoMain_MaMode,StoPriceField,0,i);   break;
           default:         buffer8[i] = iMA(NULL,0,baseMaPeriod,0,baseMaMode,PriceType,i);    
        }               
      
     }
     for(i = limit; i >= 0 ; i--)
     {
        if (MACDType == macd_onZlag)
        {
           buffer5[i] = iCustom(NULL,0,IndicatorName,"DoingZeroLag",FastEMA,SlowEMA,SignalEMA,PriceType,0,i);
           buffer6[i] = iCustom(NULL,0,IndicatorName,"DoingZeroLag",FastEMA,SlowEMA,SignalEMA,PriceType,1,i);
        }               
        else
        {
         buffer5[i] = iMAOnArray(buffer8,0,FastEMA,0,MODE_EMA,i)-iMAOnArray(buffer8,0,SlowEMA,0,MODE_EMA,i);
         buffer6[i] = buffer6[i+1]+ signalAlpha*(buffer5[i]-buffer6[i+1]);
        }            

      //
      //
      //
      //
      //
                          
      buffer1[i] = buffer2[i]= EMPTY_VALUE;
      buffer3[i] = buffer4[i]= EMPTY_VALUE;
      slope[i]   = slope[i+1];
      if (!ShowOSMA)
      {
         if (buffer5[i]>0) 
         {
            if (buffer5[i]>buffer5[i+1]) 
                  buffer2[i]=buffer5[i];
            else  buffer1[i]=buffer5[i];
         }            
         else    
         {         
            if (buffer5[i]<buffer5[i+1]) 
                  buffer4[i]=buffer5[i];
            else  buffer3[i]=buffer5[i];
         }
            if (buffer5[i]>buffer5[i+1]) slope[i] =  1;
            if (buffer5[i]<buffer5[i+1]) slope[i] = -1;
            manageArrow(i);
         }
       else
       {
          buffer7[i] = (buffer5[i]-buffer6[i])*OsmaMultiplier;
          if (buffer7[i]>0) 
               {
                  if (buffer7[i]>buffer7[i+1]) 
                        buffer2[i]=buffer7[i];
                  else  buffer1[i]=buffer7[i];
               }            
            else    
               {         
                  if (buffer7[i]<buffer7[i+1]) 
                        buffer4[i]=buffer7[i];
                  else  buffer3[i]=buffer7[i];
              }
            if (buffer7[i]>buffer7[i+1]) slope[i] =  1;
            if (buffer7[i]<buffer7[i+1]) slope[i] = -1;
            manageArrow(i);
         }
      }
   return(0);
   }      

   //
   //
   //
   //
   //
   
   limit = MathMax(limit,timeFrame/_Period);
   for(i = limit; i >= 0 ; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         buffer1[i] = iCustom(NULL,timeFrame,IndicatorName,"calculating",FastEMA,SlowEMA,SignalEMA,PriceType,ShowHisto,ShowOSMA,OsmaMultiplier,ShowLines,MACDType,RSIPeriod,MOMPeriod,CCIPeriod,StoK_period,StoD_period,StoSl_period,StoMain_MaMode,StoPriceField,0,y);
         buffer2[i] = iCustom(NULL,timeFrame,IndicatorName,"calculating",FastEMA,SlowEMA,SignalEMA,PriceType,ShowHisto,ShowOSMA,OsmaMultiplier,ShowLines,MACDType,RSIPeriod,MOMPeriod,CCIPeriod,StoK_period,StoD_period,StoSl_period,StoMain_MaMode,StoPriceField,1,y);
         buffer3[i] = iCustom(NULL,timeFrame,IndicatorName,"calculating",FastEMA,SlowEMA,SignalEMA,PriceType,ShowHisto,ShowOSMA,OsmaMultiplier,ShowLines,MACDType,RSIPeriod,MOMPeriod,CCIPeriod,StoK_period,StoD_period,StoSl_period,StoMain_MaMode,StoPriceField,2,y);
         buffer4[i] = iCustom(NULL,timeFrame,IndicatorName,"calculating",FastEMA,SlowEMA,SignalEMA,PriceType,ShowHisto,ShowOSMA,OsmaMultiplier,ShowLines,MACDType,RSIPeriod,MOMPeriod,CCIPeriod,StoK_period,StoD_period,StoSl_period,StoMain_MaMode,StoPriceField,3,y);
         buffer5[i] = iCustom(NULL,timeFrame,IndicatorName,"calculating",FastEMA,SlowEMA,SignalEMA,PriceType,ShowHisto,ShowOSMA,OsmaMultiplier,ShowLines,MACDType,RSIPeriod,MOMPeriod,CCIPeriod,StoK_period,StoD_period,StoSl_period,StoMain_MaMode,StoPriceField,4,y);
         buffer6[i] = iCustom(NULL,timeFrame,IndicatorName,"calculating",FastEMA,SlowEMA,SignalEMA,PriceType,ShowHisto,ShowOSMA,OsmaMultiplier,ShowLines,MACDType,RSIPeriod,MOMPeriod,CCIPeriod,StoK_period,StoD_period,StoSl_period,StoMain_MaMode,StoPriceField,5,y);
         buffer7[i] = iCustom(NULL,timeFrame,IndicatorName,"calculating",FastEMA,SlowEMA,SignalEMA,PriceType,ShowHisto,ShowOSMA,OsmaMultiplier,ShowLines,MACDType,RSIPeriod,MOMPeriod,CCIPeriod,StoK_period,StoD_period,StoSl_period,StoMain_MaMode,StoPriceField,6,y);
         slope[i] = slope[i+1];
         if (!ShowOSMA)
         {
         if (buffer5[i]>0) 
         {
            if (buffer5[i]>buffer5[i+1]) 
                  buffer2[i]=buffer5[i];
            else  buffer1[i]=buffer5[i];
         }            
         else    
         {         
            if (buffer5[i]<buffer5[i+1]) 
                  buffer4[i]=buffer5[i];
            else  buffer3[i]=buffer5[i];
         }
            if (buffer5[i]>buffer5[i+1]) slope[i] =  1;
            if (buffer5[i]<buffer5[i+1]) slope[i] = -1;
            manageArrow(i);
         }
       else
       {
          buffer7[i] = (buffer5[i]-buffer6[i])*OsmaMultiplier;
          if (buffer7[i]>0) 
               {
                  if (buffer7[i]>buffer7[i+1]) 
                        buffer2[i]=buffer7[i];
                  else  buffer1[i]=buffer7[i];
               }            
            else    
               {         
                  if (buffer7[i]<buffer7[i+1]) 
                        buffer4[i]=buffer7[i];
                  else  buffer3[i]=buffer7[i];
              }
            if (buffer7[i]>buffer7[i+1]) slope[i] =  1;
            if (buffer7[i]<buffer7[i+1]) slope[i] = -1;
            manageArrow(i);
         }
    }
    
   //
   //
   //
   //
   //

   if (alertsOn)
   {
      //
      //
      //
      //
      //
     
     if(MACDalerts)
      {       
      if (buffer5[0]>0 && buffer5[1]<0) doAlert("macd line crossed 0 up "); 
      if (buffer5[0]<0 && buffer5[1]>0) doAlert("macd line crossed 0 down "); 
      }
     
     if (OsMAalerts||MACDalerts)
      { 
      if (buffer6[0]<buffer5[0] && buffer6[1] > buffer5[1]) doAlert("osma crossed 0 (macd crossed signal line) up "); 
      if (buffer6[0]>buffer5[0] && buffer6[1] < buffer5[1]) doAlert("osma crossed 0 (macd crossed signal line) down "); 
      }
     
     if (OsMAalerts)
      {
      if (buffer7[0]>buffer7[1] && buffer7[2] > buffer7[1]) doAlert("osma macd(histo)changed direction: up "); 
      if (buffer7[0]<buffer7[1] && buffer7[2] < buffer7[1]) doAlert("osma macd(histo)changed direction: down "); 
      }


   }   
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrow(int i)
{
    if (arrowsVisible)
       {
          string lookFor = arrowsIdentifier+":"+(string)Time[i]; ObjectDelete(lookFor);            
          if (i<(Bars-1) && slope[i] != slope[i+1])
          {
              if (slope[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,arrowsUpSize,false);
              if (slope[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode,arrowsDnSize, true);
          }
       } 
}                  

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void drawArrow(int i,color theColor,int theCode, int theSize, bool up)
{
   string name = arrowsIdentifier+":"+(string)Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //

      datetime atime = Time[i]; if (arrowsOnNewest) atime += _Period*60-1;      
      ObjectCreate(name,OBJ_ARROW,0,atime,0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_WIDTH,theSize);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CalculateZeroLag(int limit)
{
   int    i;
   double maf,mas;
   
   //
   //
   //
   //
   //
   
   for(i=limit;i>=0;i--)
   {
      buffer3[i]=iMA(NULL,0,FastEMA,0,MODE_EMA,PriceType,i);
      buffer4[i]=iMA(NULL,0,SlowEMA,0,MODE_EMA,PriceType,i);
   }
   for(i=limit;i>=0;i--)
   {
      maf=buffer3[i]+buffer3[i]-iMAOnArray(buffer3,0,FastEMA,0,MODE_EMA,i);
      mas=buffer4[i]+buffer4[i]-iMAOnArray(buffer4,0,SlowEMA,0,MODE_EMA,i);
          buffer1[i]=maf-mas;
   }
   
   //
   //
   //
   //
   //
   
   for(i=limit;i>=0;i--) buffer5[i]=                      iMAOnArray(buffer1,0,SignalEMA,0,MODE_EMA,i);
   for(i=limit;i>=0;i--) buffer2[i]=buffer5[i]+buffer5[i]-iMAOnArray(buffer5,0,SignalEMA,0,MODE_EMA,i);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void doAlert(string forWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != forWhat || previousTime != Time[0]) {
       previousAlert  = forWhat;
       previousTime   = Time[0];
       message        = StringConcatenate(Symbol()," ",forWhat," at ",TimeToStr(TimeLocal(),TIME_SECONDS));
           if (alertsMessage) Alert(message);
           if (alertsSound)   PlaySound(soundfile);
           if (alertsEmail)   SendMail(StringConcatenate(Symbol()," MACD crossing"),message);
   }           
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}     

